# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["AISource"]


class AISource(BaseModel):
    """A source cited in an AI-generated search answer."""

    index: int
    """1-based citation index corresponding to [N] references in the answer text."""

    node_id: str
    """Unique identifier of the source node in the search tree."""

    title: str
    """Human-readable title of the source."""

    type: Literal["file", "record", "table"]
    """
    The type of source: `file` (document/attachment), `record` (data row), or
    `table` (dataset).
    """

    dataset_name: Optional[str] = None
    """Name of the dataset this source belongs to, if applicable."""

    emoji: Optional[str] = None
    """Emoji icon for the source type (e.g., 📄 for files, 📊 for tables)."""

    page_numbers: Optional[List[int]] = None
    """Page numbers within a document where the relevant content was found."""

    table_name: Optional[str] = None
    """Name of the table this source belongs to, if applicable."""

    thumbnail_uri: Optional[str] = None
    """URI for a thumbnail preview of the source, if available."""

    url: Optional[str] = None
    """URL to view the source in the DataGrid web app."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .user import (
    UserResource,
    AsyncUserResource,
    UserResourceWithRawResponse,
    AsyncUserResourceWithRawResponse,
    UserResourceWithStreamingResponse,
    AsyncUserResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["MemoryResource", "AsyncMemoryResource"]


class MemoryResource(SyncAPIResource):
    @cached_property
    def user(self) -> UserResource:
        return UserResource(self._client)

    @cached_property
    def with_raw_response(self) -> MemoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return MemoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MemoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return MemoryResourceWithStreamingResponse(self)


class AsyncMemoryResource(AsyncAPIResource):
    @cached_property
    def user(self) -> AsyncUserResource:
        return AsyncUserResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMemoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMemoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMemoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AsyncMemoryResourceWithStreamingResponse(self)


class MemoryResourceWithRawResponse:
    def __init__(self, memory: MemoryResource) -> None:
        self._memory = memory

    @cached_property
    def user(self) -> UserResourceWithRawResponse:
        return UserResourceWithRawResponse(self._memory.user)


class AsyncMemoryResourceWithRawResponse:
    def __init__(self, memory: AsyncMemoryResource) -> None:
        self._memory = memory

    @cached_property
    def user(self) -> AsyncUserResourceWithRawResponse:
        return AsyncUserResourceWithRawResponse(self._memory.user)


class MemoryResourceWithStreamingResponse:
    def __init__(self, memory: MemoryResource) -> None:
        self._memory = memory

    @cached_property
    def user(self) -> UserResourceWithStreamingResponse:
        return UserResourceWithStreamingResponse(self._memory.user)


class AsyncMemoryResourceWithStreamingResponse:
    def __init__(self, memory: AsyncMemoryResource) -> None:
        self._memory = memory

    @cached_property
    def user(self) -> AsyncUserResourceWithStreamingResponse:
        return AsyncUserResourceWithStreamingResponse(self._memory.user)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["FileCreateParams"]


class FileCreateParams(TypedDict, total=False):
    file: Required[FileTypes]

    expires_after: Optional[int]
    """
    The number of seconds after creation when the file will expire and be
    automatically deleted. Must be a positive integer, maximum 6 days (518400s). If
    not provided, the file will not expire.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ServiceAccount"]


class ServiceAccount(BaseModel):
    """
    The `service_account` object represents a service account for accessing data views.
    """

    id: str
    """The service account identifier."""

    created_at: datetime
    """The ISO string for when the service account was created."""

    email: str
    """The email address of the service account."""

    object: Literal["service_account"]
    """The object type, which is always `service_account`."""

    type: Literal["gcp"]
    """The type of service account, currently only `gcp` is supported."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["SearchSearchParams"]


class SearchSearchParams(TypedDict, total=False):
    query: Required[str]
    """The search query to run against knowledge.

    Must contain at least one non-whitespace character after trimming.
    """

    knowledge_ids: SequenceNotStr[str]
    """Optional list of knowledge IDs to scope the search to."""

    limit: int
    """The limit on the number of objects to return, ranging between 1 and 100."""

    next: str
    """A cursor to use in pagination to continue a query from the previous request.

    This is automatically added when the request has more results to fetch.
    """

    record_types: List[Literal["rows", "tables", "files", "pages", "cells"]]
    """Filter results by record type in the vector database."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["IdentityTeamspace", "Permissions"]


class Permissions(BaseModel):
    """The permissions the user holds in this teamspace."""

    role: Literal["owner", "admin", "member", "collaborator", "agents-only", "agent-specific"]
    """The role assigned to the user. Available roles:

    - **owner**: Creator of the teamspace. Full control over the teamspace. Can
      manage all users, settings, and resources.
    - **admin**: Full control over the teamspace. Can manage all users, settings,
      and resources.
    - **member**: Standard member access. Can view and interact with teamspace
      resources. Can invite other members.
    - **collaborator**: Read-only access with ability to comment and provide
      feedback.
    - **agents-only**: Access limited to AI agent interactions within the teamspace.
    - **agent-specific**: Limited access on teamspace level, can only access agents
      that are assigned to the teamspace.
    """

    agent_ids: Optional[List[str]] = None
    """The agent IDs that the user has access to, if the role is agent-specific."""


class IdentityTeamspace(BaseModel):
    permissions: Permissions
    """The permissions the user holds in this teamspace."""

    teamspace_id: str
    """The ID of the teamspace."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["UserUpdateParams"]


class UserUpdateParams(TypedDict, total=False):
    teamspace_id: Required[str]

    role: Required[Literal["admin", "member", "agents-only", "agent-specific"]]

    agent_ids: Optional[SequenceNotStr[str]]
    """The agent IDs that the user has access to, if the role is agent-specific."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .data_view import DataView

__all__ = ["DataViewCreateResponse", "DataViewCreateResponseCredits"]


class DataViewCreateResponseCredits(BaseModel):
    """Credit consumption for this data-view creation.

    `consumed` reflects the warehouse setup work billed for the request. `null` when the billing write fails after the data view is successfully created.
    """

    consumed: float
    """The number of credits consumed by the operation."""


class DataViewCreateResponse(DataView):
    """
    The `data_view` object represents a view into a knowledge source that can be accessed through a service account.
    """

    credits: Optional[DataViewCreateResponseCredits] = None
    """Credit consumption for this data-view creation.

    `consumed` reflects the warehouse setup work billed for the request. `null` when
    the billing write fails after the data view is successfully created.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel
from .search_result_item import SearchResultItem

__all__ = ["SearchSearchResponse", "Credits"]


class Credits(BaseModel):
    """Credit consumption for this search.

    When present, `consumed` reflects the actual variable cost of the retrieval work performed for this request. `null` when the billing write fails or the request is aborted before billing completes.
    """

    consumed: float
    """The number of credits consumed by the operation."""


class SearchSearchResponse(BaseModel):
    credits: Optional[Credits] = None
    """Credit consumption for this search.

    When present, `consumed` reflects the actual variable cost of the retrieval work
    performed for this request. `null` when the billing write fails or the request
    is aborted before billing completes.
    """

    data: List[SearchResultItem]
    """An array containing the found search items."""

    object: Literal["list"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DataViewCreateParams"]


class DataViewCreateParams(TypedDict, total=False):
    bigquery_dataset_name: Required[str]
    """The name of the BigQuery dataset containing views to the data.

    Your organization's domain will be automatically prepended to the name.
    """

    knowledge_id: Required[str]
    """The id of the knowledge to create a data view for."""

    service_account_id: Required[str]
    """The id of the service account that will access this data view."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["ConnectionCreateParams"]


class ConnectionCreateParams(TypedDict, total=False):
    connector_id: Required[str]
    """The connector ID for the third-party service to connect to."""

    connection_provider_id: Optional[str]
    """
    Optional connection provider ID to use custom OAuth credentials for this
    connection.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = [
    "Message",
    "Citation",
    "CitationSource",
    "Content",
    "ContentMessageContentText",
    "ContentMessageContentVoice",
    "ContentMessageContentVoiceAudioClip",
    "ContentMessageContentVoiceAudioClipParticipant",
    "ContentMessageContentVoiceTimelineEvent",
    "ContentMessageContentVoiceTimelineEventCitation",
    "ContentMessageContentVoiceTimelineEventCitationSource",
    "ContentMessageContentFile",
    "Credits",
]


class CitationSource(BaseModel):
    confirmations: List[str]
    """An array of text snippets from the source that confirm the citation."""

    source_name: str
    """Name of the source."""

    type: Literal["image", "pdf_page", "record", "web_search", "sql_query_result", "action"]

    source_id: Optional[str] = None
    """Id of the source."""

    source_uri: Optional[str] = None
    """URI of the source."""


class Citation(BaseModel):
    citation: str
    """The text snippet from the response that is being cited."""

    sources: List[CitationSource]
    """Array of sources that support this citation."""


class ContentMessageContentText(BaseModel):
    """Text content for a message."""

    text: str
    """The text content of the message."""

    type: Literal["text"]


class ContentMessageContentVoiceAudioClipParticipant(BaseModel):
    """Participant who spoke in this clip."""

    id: str

    type: Literal["user", "agent"]


class ContentMessageContentVoiceAudioClip(BaseModel):
    """A single audio clip from a voice message."""

    id: str
    """Unique identifier for the audio clip file.

    Use this ID with the files API to download the audio content.
    """

    audio_uri: str
    """Datagrid file URI for the audio file (WAV format)."""

    duration_ms: float
    """Duration of this audio clip in milliseconds."""

    participant: ContentMessageContentVoiceAudioClipParticipant
    """Participant who spoke in this clip."""

    start_time_ms: float
    """
    Start time of this clip relative to the beginning of the voice message, in
    milliseconds.
    """


class ContentMessageContentVoiceTimelineEventCitationSource(BaseModel):
    confirmations: List[str]
    """An array of text snippets from the source that confirm the citation."""

    source_name: str
    """Name of the source."""

    type: Literal["image", "pdf_page", "record", "web_search", "sql_query_result", "action"]

    source_id: Optional[str] = None
    """Id of the source."""

    source_uri: Optional[str] = None
    """URI of the source."""


class ContentMessageContentVoiceTimelineEventCitation(BaseModel):
    citation: str
    """The text snippet from the response that is being cited."""

    sources: List[ContentMessageContentVoiceTimelineEventCitationSource]
    """Array of sources that support this citation."""


class ContentMessageContentVoiceTimelineEvent(BaseModel):
    """
    A single event from a voice session timeline, representing either a transcript turn or a citation.
    """

    timestamp_ms: float
    """Timestamp offset from the start of the voice session, in milliseconds."""

    type: Literal["transcript", "citation"]
    """The type of timeline event."""

    citations: Optional[List[ContentMessageContentVoiceTimelineEventCitation]] = None
    """Citations for this event. Present when type is 'citation'."""

    role: Optional[Literal["user", "agent"]] = None
    """The role of the participant for this event."""

    text: Optional[str] = None
    """Plain text transcript for this turn. Present when type is 'transcript'."""


class ContentMessageContentVoice(BaseModel):
    """Voice content for a message."""

    audio_clips: List[ContentMessageContentVoiceAudioClip]
    """Array of audio clips with timestamps for synchronized playback."""

    duration_ms: Optional[float] = None
    """Total duration of the voice message in milliseconds."""

    transcript: str
    """User transcript of the voice message."""

    type: Literal["voice"]

    agent_transcript: Optional[str] = None
    """Agent transcript of the voice message (for agent role messages)."""

    timeline_events: Optional[List[ContentMessageContentVoiceTimelineEvent]] = None
    """Per-turn transcript and citation events in chronological order.

    Each entry is plain text with a timestamp offset from the start of the voice
    session. Present only for voice sessions that recorded timeline data.
    """


class ContentMessageContentFile(BaseModel):
    """File attachment content for a message.

    Represents a file that was uploaded as part of the user's message.
    """

    file_id: str
    """The ID of the attached file.

    Use this ID with the files API to download the file content.
    """

    type: Literal["input_file"]


Content: TypeAlias = Annotated[
    Union[ContentMessageContentText, ContentMessageContentVoice, ContentMessageContentFile],
    PropertyInfo(discriminator="type"),
]


class Credits(BaseModel):
    """Credit consumption for this converse turn.

    `null` for user-role messages and when retrieving messages from conversation history.
    """

    consumed: float
    """The number of credits consumed by the operation."""


class Message(BaseModel):
    """The `conversation.message` object represents a message in a conversation."""

    id: str
    """The message identifier."""

    agent_id: str
    """The ID of the agent that sent or responded to the message."""

    citations: Optional[List[Citation]] = None
    """Array of citations that provide sources for factual statements in the response.

    Each citation includes the referenced text and its sources.
    """

    content: List[Content]
    """Contents of the message."""

    conversation_id: str
    """The ID of the conversation the message belongs to."""

    created_at: datetime
    """The ISO string for when the message was created."""

    credits: Optional[Credits] = None
    """Credit consumption for this converse turn.

    `null` for user-role messages and when retrieving messages from conversation
    history.
    """

    object: Literal["conversation.message"]
    """The object type, which is always `conversation.message`."""

    role: Literal["user", "agent"]
    """The role of the message sender - either 'user' or 'agent'."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Mapping, Optional, cast
from typing_extensions import Literal

import httpx

from ...types import knowledge_list_params, knowledge_create_params, knowledge_update_params, knowledge_connect_params
from ..._types import (
    Body,
    Omit,
    Query,
    Headers,
    NoneType,
    NotGiven,
    FileTypes,
    SequenceNotStr,
    omit,
    not_given,
)
from ..._utils import is_given, extract_files, path_template, maybe_transform, deepcopy_minimal, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._constants import DEFAULT_TIMEOUT
from ...pagination import SyncCursorIDPage, AsyncCursorIDPage
from .tables.tables import (
    TablesResource,
    AsyncTablesResource,
    TablesResourceWithRawResponse,
    AsyncTablesResourceWithRawResponse,
    TablesResourceWithStreamingResponse,
    AsyncTablesResourceWithStreamingResponse,
)
from ..._base_client import AsyncPaginator, make_request_options
from ...types.knowledge.knowledge import Knowledge
from ...types.redirect_url_response import RedirectURLResponse

__all__ = ["KnowledgeResource", "AsyncKnowledgeResource"]


class KnowledgeResource(SyncAPIResource):
    @cached_property
    def tables(self) -> TablesResource:
        return TablesResource(self._client)

    @cached_property
    def with_raw_response(self) -> KnowledgeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return KnowledgeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> KnowledgeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return KnowledgeResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        files: SequenceNotStr[FileTypes],
        name: Optional[str] | Omit = omit,
        parent: Optional[knowledge_create_params.Parent] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Knowledge:
        """
        Create knowledge which will be learned and leveraged by agents.

        Args:
          files: The files to be uploaded and learned. Supported media types are `pdf`, `json`,
              `csv`, `text`, `png`, `jpeg`, `excel`, `google sheets`, `docx`, `pptx`.

          name: The name of the knowledge.

          parent: The parent page to nest this knowledge under. If not provided, knowledge will be
              created at the root level.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not is_given(timeout) and self._client.timeout == DEFAULT_TIMEOUT:
            timeout = 300
        body = deepcopy_minimal(
            {
                "files": files,
                "name": name,
                "parent": parent,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/knowledge",
            body=maybe_transform(body, knowledge_create_params.KnowledgeCreateParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Knowledge,
        )

    def retrieve(
        self,
        knowledge_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Knowledge:
        """
        Retrieves knowledge by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_id` but received {knowledge_id!r}")
        return self._get(
            path_template("/knowledge/{knowledge_id}", knowledge_id=knowledge_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Knowledge,
        )

    def update(
        self,
        knowledge_id: str,
        *,
        files: Optional[SequenceNotStr[FileTypes]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        parent: Optional[knowledge_update_params.Parent] | Omit = omit,
        scope: Optional[Literal["teamspace", "organization"]] | Omit = omit,
        sync: Optional[knowledge_update_params.Sync] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Knowledge:
        """Update a knowledge's attributes.

        Each request can include either `files` or
        `sync`, but not both. When `files` are provided, all existing data is replaced
        and a re-processing pipeline runs asynchronously — this consumes credits based
        on the volume of data processed. Metadata-only and sync-only updates do not
        consume credits and are not blocked by credit eligibility checks.

        Args:
          files: The files to replace existing knowledge. When provided, all existing data will
              be removed from the knowledge and replaced with these files. Supported media
              types are `pdf`, `json`, `csv`, `text`, `png`, `jpeg`, `excel`, `google sheets`,
              `docx`, `pptx`. Cannot be used together with `sync` in the same request.

          name: The new name for the `knowledge`.

          parent: Move the knowledge to a different parent page.

          scope: The visibility scope of the knowledge. 'teamspace' means visible only within the
              owning teamspace. 'organization' means visible across all teamspaces in the same
              organization.

          sync: Sync configuration updates for knowledge created from a connection. Note: For
              multipart/form-data, this should be sent as a JSON string. Cannot be used
              together with `files` in the same request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_id` but received {knowledge_id!r}")
        body = deepcopy_minimal(
            {
                "files": files,
                "name": name,
                "parent": parent,
                "scope": scope,
                "sync": sync,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._patch(
            path_template("/knowledge/{knowledge_id}", knowledge_id=knowledge_id),
            body=maybe_transform(body, knowledge_update_params.KnowledgeUpdateParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Knowledge,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        limit: int | Omit = omit,
        parent: knowledge_list_params.Parent | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorIDPage[Knowledge]:
        """Returns a list of knowledge.

        Args:
          after: A cursor to use in pagination.

        `after` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              ending with `obj_foo`, your subsequent call can include `after=obj_foo` to fetch
              the next page of the list.

          before: A cursor to use in pagination. `before` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              starting with `obj_bar`, your subsequent call can include `before=obj_bar` to
              fetch the previous page of the list.

          limit: The limit on the number of objects to return, ranging between 1 and 100.

          parent: Filter by parent. Pass `{"type":"root"}` to get root-level items, or
              `{"type":"page","page_id":"page_123"}` to get items nested under a specific
              page. If not specified, returns all items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/knowledge",
            page=SyncCursorIDPage[Knowledge],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "parent": parent,
                    },
                    knowledge_list_params.KnowledgeListParams,
                ),
            ),
            model=Knowledge,
        )

    def delete(
        self,
        knowledge_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete knowledge.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_id` but received {knowledge_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/knowledge/{knowledge_id}", knowledge_id=knowledge_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def connect(
        self,
        *,
        connection_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedirectURLResponse:
        """Initiates knowledge creation from a connection by returning a redirect URL.

        The
        organization must have enough credits to start this flow. The downstream
        ingestion and indexing that follow still run asynchronously, and the actual
        credit consumption remains variable based on the volume of data processed.

        Args:
          connection_id: The id of the connection to be used to create the knowledge.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/knowledge/connect",
            body=maybe_transform({"connection_id": connection_id}, knowledge_connect_params.KnowledgeConnectParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedirectURLResponse,
        )

    def reindex(
        self,
        knowledge_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Manually trigger a full re-indexing of the knowledge.

        The reindex runs
        **asynchronously**: the API returns as soon as the job is enqueued. Re-indexing
        is not performed immediately. This endpoint consumes credits — the actual credit
        cost is variable, based on the volume of data being re-indexed, and is charged
        asynchronously as processing completes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_id` but received {knowledge_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            path_template("/knowledge/{knowledge_id}/reindex", knowledge_id=knowledge_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncKnowledgeResource(AsyncAPIResource):
    @cached_property
    def tables(self) -> AsyncTablesResource:
        return AsyncTablesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncKnowledgeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AsyncKnowledgeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncKnowledgeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AsyncKnowledgeResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        files: SequenceNotStr[FileTypes],
        name: Optional[str] | Omit = omit,
        parent: Optional[knowledge_create_params.Parent] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Knowledge:
        """
        Create knowledge which will be learned and leveraged by agents.

        Args:
          files: The files to be uploaded and learned. Supported media types are `pdf`, `json`,
              `csv`, `text`, `png`, `jpeg`, `excel`, `google sheets`, `docx`, `pptx`.

          name: The name of the knowledge.

          parent: The parent page to nest this knowledge under. If not provided, knowledge will be
              created at the root level.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not is_given(timeout) and self._client.timeout == DEFAULT_TIMEOUT:
            timeout = 300
        body = deepcopy_minimal(
            {
                "files": files,
                "name": name,
                "parent": parent,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/knowledge",
            body=await async_maybe_transform(body, knowledge_create_params.KnowledgeCreateParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Knowledge,
        )

    async def retrieve(
        self,
        knowledge_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Knowledge:
        """
        Retrieves knowledge by id.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_id` but received {knowledge_id!r}")
        return await self._get(
            path_template("/knowledge/{knowledge_id}", knowledge_id=knowledge_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Knowledge,
        )

    async def update(
        self,
        knowledge_id: str,
        *,
        files: Optional[SequenceNotStr[FileTypes]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        parent: Optional[knowledge_update_params.Parent] | Omit = omit,
        scope: Optional[Literal["teamspace", "organization"]] | Omit = omit,
        sync: Optional[knowledge_update_params.Sync] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Knowledge:
        """Update a knowledge's attributes.

        Each request can include either `files` or
        `sync`, but not both. When `files` are provided, all existing data is replaced
        and a re-processing pipeline runs asynchronously — this consumes credits based
        on the volume of data processed. Metadata-only and sync-only updates do not
        consume credits and are not blocked by credit eligibility checks.

        Args:
          files: The files to replace existing knowledge. When provided, all existing data will
              be removed from the knowledge and replaced with these files. Supported media
              types are `pdf`, `json`, `csv`, `text`, `png`, `jpeg`, `excel`, `google sheets`,
              `docx`, `pptx`. Cannot be used together with `sync` in the same request.

          name: The new name for the `knowledge`.

          parent: Move the knowledge to a different parent page.

          scope: The visibility scope of the knowledge. 'teamspace' means visible only within the
              owning teamspace. 'organization' means visible across all teamspaces in the same
              organization.

          sync: Sync configuration updates for knowledge created from a connection. Note: For
              multipart/form-data, this should be sent as a JSON string. Cannot be used
              together with `files` in the same request.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_id` but received {knowledge_id!r}")
        body = deepcopy_minimal(
            {
                "files": files,
                "name": name,
                "parent": parent,
                "scope": scope,
                "sync": sync,
            }
        )
        extracted_files = extract_files(cast(Mapping[str, object], body), paths=[["files", "<array>"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._patch(
            path_template("/knowledge/{knowledge_id}", knowledge_id=knowledge_id),
            body=await async_maybe_transform(body, knowledge_update_params.KnowledgeUpdateParams),
            files=extracted_files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Knowledge,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        limit: int | Omit = omit,
        parent: knowledge_list_params.Parent | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Knowledge, AsyncCursorIDPage[Knowledge]]:
        """Returns a list of knowledge.

        Args:
          after: A cursor to use in pagination.

        `after` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              ending with `obj_foo`, your subsequent call can include `after=obj_foo` to fetch
              the next page of the list.

          before: A cursor to use in pagination. `before` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              starting with `obj_bar`, your subsequent call can include `before=obj_bar` to
              fetch the previous page of the list.

          limit: The limit on the number of objects to return, ranging between 1 and 100.

          parent: Filter by parent. Pass `{"type":"root"}` to get root-level items, or
              `{"type":"page","page_id":"page_123"}` to get items nested under a specific
              page. If not specified, returns all items.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/knowledge",
            page=AsyncCursorIDPage[Knowledge],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "parent": parent,
                    },
                    knowledge_list_params.KnowledgeListParams,
                ),
            ),
            model=Knowledge,
        )

    async def delete(
        self,
        knowledge_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete knowledge.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_id` but received {knowledge_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/knowledge/{knowledge_id}", knowledge_id=knowledge_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def connect(
        self,
        *,
        connection_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RedirectURLResponse:
        """Initiates knowledge creation from a connection by returning a redirect URL.

        The
        organization must have enough credits to start this flow. The downstream
        ingestion and indexing that follow still run asynchronously, and the actual
        credit consumption remains variable based on the volume of data processed.

        Args:
          connection_id: The id of the connection to be used to create the knowledge.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/knowledge/connect",
            body=await async_maybe_transform(
                {"connection_id": connection_id}, knowledge_connect_params.KnowledgeConnectParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RedirectURLResponse,
        )

    async def reindex(
        self,
        knowledge_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Manually trigger a full re-indexing of the knowledge.

        The reindex runs
        **asynchronously**: the API returns as soon as the job is enqueued. Re-indexing
        is not performed immediately. This endpoint consumes credits — the actual credit
        cost is variable, based on the volume of data being re-indexed, and is charged
        asynchronously as processing completes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not knowledge_id:
            raise ValueError(f"Expected a non-empty value for `knowledge_id` but received {knowledge_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            path_template("/knowledge/{knowledge_id}/reindex", knowledge_id=knowledge_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class KnowledgeResourceWithRawResponse:
    def __init__(self, knowledge: KnowledgeResource) -> None:
        self._knowledge = knowledge

        self.create = to_raw_response_wrapper(
            knowledge.create,
        )
        self.retrieve = to_raw_response_wrapper(
            knowledge.retrieve,
        )
        self.update = to_raw_response_wrapper(
            knowledge.update,
        )
        self.list = to_raw_response_wrapper(
            knowledge.list,
        )
        self.delete = to_raw_response_wrapper(
            knowledge.delete,
        )
        self.connect = to_raw_response_wrapper(
            knowledge.connect,
        )
        self.reindex = to_raw_response_wrapper(
            knowledge.reindex,
        )

    @cached_property
    def tables(self) -> TablesResourceWithRawResponse:
        return TablesResourceWithRawResponse(self._knowledge.tables)


class AsyncKnowledgeResourceWithRawResponse:
    def __init__(self, knowledge: AsyncKnowledgeResource) -> None:
        self._knowledge = knowledge

        self.create = async_to_raw_response_wrapper(
            knowledge.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            knowledge.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            knowledge.update,
        )
        self.list = async_to_raw_response_wrapper(
            knowledge.list,
        )
        self.delete = async_to_raw_response_wrapper(
            knowledge.delete,
        )
        self.connect = async_to_raw_response_wrapper(
            knowledge.connect,
        )
        self.reindex = async_to_raw_response_wrapper(
            knowledge.reindex,
        )

    @cached_property
    def tables(self) -> AsyncTablesResourceWithRawResponse:
        return AsyncTablesResourceWithRawResponse(self._knowledge.tables)


class KnowledgeResourceWithStreamingResponse:
    def __init__(self, knowledge: KnowledgeResource) -> None:
        self._knowledge = knowledge

        self.create = to_streamed_response_wrapper(
            knowledge.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            knowledge.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            knowledge.update,
        )
        self.list = to_streamed_response_wrapper(
            knowledge.list,
        )
        self.delete = to_streamed_response_wrapper(
            knowledge.delete,
        )
        self.connect = to_streamed_response_wrapper(
            knowledge.connect,
        )
        self.reindex = to_streamed_response_wrapper(
            knowledge.reindex,
        )

    @cached_property
    def tables(self) -> TablesResourceWithStreamingResponse:
        return TablesResourceWithStreamingResponse(self._knowledge.tables)


class AsyncKnowledgeResourceWithStreamingResponse:
    def __init__(self, knowledge: AsyncKnowledgeResource) -> None:
        self._knowledge = knowledge

        self.create = async_to_streamed_response_wrapper(
            knowledge.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            knowledge.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            knowledge.update,
        )
        self.list = async_to_streamed_response_wrapper(
            knowledge.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            knowledge.delete,
        )
        self.connect = async_to_streamed_response_wrapper(
            knowledge.connect,
        )
        self.reindex = async_to_streamed_response_wrapper(
            knowledge.reindex,
        )

    @cached_property
    def tables(self) -> AsyncTablesResourceWithStreamingResponse:
        return AsyncTablesResourceWithStreamingResponse(self._knowledge.tables)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["DataView"]


class DataView(BaseModel):
    """
    The `data_view` object represents a view into a knowledge source that can be accessed through a service account.
    """

    id: str
    """The data view identifier."""

    bigquery_dataset_name: str
    """The name of the BigQuery dataset containing views to the data."""

    created_at: datetime
    """The ISO string for when the data view was created."""

    knowledge_id: str
    """The id of the knowledge this data view is for."""

    object: Literal["data_view"]
    """The object type, which is always `data_view`."""

    service_account_id: str
    """The id of the service account that can access this data view."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["ToolDef"]


class ToolDef(BaseModel):
    """The `Tool` object represents a tool that can be used by agents."""

    description: str
    """A detailed description of what the tool does."""

    label: str
    """The display name of the tool."""

    name: str
    """The unique identifier name used to reference the tool."""

    object: Literal["tool"]
    """The object type, which is always `tool`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = ["Knowledge", "Credits", "Parent", "ParentParentPage", "ParentRootPage", "RowCounts", "Sync", "SyncTrigger"]


class Credits(BaseModel):
    """Credit consumption for this knowledge.

    `null` when the knowledge is still being processed and the final cost is not yet known (e.g. immediately after creation or reindexing), or when the credit lookup fails. When present, `consumed` is the current summed spend for the knowledge's active tables — it is not necessarily the cost of the most recent request, nor the full lifetime spend.
    """

    consumed: float
    """The number of credits consumed by the operation."""


class ParentParentPage(BaseModel):
    """The parent page reference, indicating where this page is nested"""

    page_id: str
    """The ID of the parent page. Required when type is 'page'"""

    type: Literal["page"]
    """The type of parent. 'page' indicates nested under a specific page"""


class ParentRootPage(BaseModel):
    """The root level object"""

    type: Literal["root"]
    """The type of parent. 'root' indicates at the root level"""


Parent: TypeAlias = Union[ParentParentPage, ParentRootPage]


class RowCounts(BaseModel):
    """Row count statistics for the knowledge."""

    completed: float
    """The number of rows successfully learned."""

    failed: float
    """The number of rows that failed to be processed for learning."""

    total: float
    """The total number of rows in the knowledge."""


class SyncTrigger(BaseModel):
    """A cron-based schedule for syncing data"""

    cron_expression: str
    """Cron expression (e.g., '0 0 \\** \\** \\**' for daily at midnight)"""

    type: Literal["cron"]
    """The trigger type, which is always `cron`."""

    description: Optional[str] = None
    """Human-readable description of the schedule"""

    timezone: Optional[str] = None
    """IANA timezone (e.g., 'America/New_York'). Defaults to 'UTC' if not provided."""


class Sync(BaseModel):
    """Sync information for knowledge that syncs data from a connection"""

    connection_id: str
    """The ID of the connection used for syncing data to this knowledge"""

    enabled: bool
    """Whether data syncing from the connection is enabled"""

    trigger: Optional[SyncTrigger] = None
    """A cron-based schedule for syncing data"""


class Knowledge(BaseModel):
    """
    The `knowledge` object represents knowledge that an agent may leverage to respond.
    """

    id: str
    """The knowledge identifier, which can be referenced in the API endpoints."""

    created_at: datetime
    """The ISO string for when the knowledge was created."""

    credits: Optional[Credits] = None
    """Credit consumption for this knowledge.

    `null` when the knowledge is still being processed and the final cost is not yet
    known (e.g. immediately after creation or reindexing), or when the credit lookup
    fails. When present, `consumed` is the current summed spend for the knowledge's
    active tables — it is not necessarily the cost of the most recent request, nor
    the full lifetime spend.
    """

    name: str
    """The name of the knowledge."""

    object: Literal["knowledge"]
    """The object type, which is always `knowledge`."""

    parent: Parent
    """The parent object, indicating where the object is located in the hierarchy"""

    row_counts: RowCounts
    """Row count statistics for the knowledge."""

    scope: Literal["teamspace", "organization"]
    """The visibility scope of the knowledge.

    'teamspace' means visible only within the owning teamspace. 'organization' means
    visible across all teamspaces in the same organization.
    """

    status: Literal["pending", "partial", "ready"]
    """
    The current knowledge status can be one of three values: `pending`, `partial`,
    or `ready`. `pending` indicates that the knowledge is awaiting learning and will
    not be used by the agent when responding. `partial` indicates that the knowledge
    is partially learned. The agent may use some aspects of it when responding.
    `ready` indicates that the knowledge is fully learned and will be completely
    utilized in responses.
    """

    sync: Optional[Sync] = None
    """Sync information for knowledge that syncs data from a connection"""

    teamspace_id: str
    """The ID of the teamspace that owns this knowledge."""

    updated_at: Optional[datetime] = None
    """The ISO string for when the knowledge was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["McpServer", "Metadata"]


class Metadata(BaseModel):
    """Safe subset of server metadata exposed to API consumers."""

    last_synced_at: Optional[str] = None

    oauth_configured: Optional[bool] = None

    requires_oauth: Optional[bool] = None

    tool_count: Optional[int] = None

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, object] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> object: ...
    else:
        __pydantic_extra__: Dict[str, object]


class McpServer(BaseModel):
    id: str

    authorization_secret_id: Optional[str] = None
    """
    Secret ID containing the full Authorization header value used for this
    registered MCP server.
    """

    base_url: str

    created_at: datetime

    icon_url: Optional[str] = None

    metadata: Optional[Metadata] = None
    """Safe subset of server metadata exposed to API consumers."""

    name: str

    object: Literal["mcp_server"]

    protocol_version: Optional[str] = None

    status: str

    transport: str

    updated_at: datetime



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["UserCreateParams"]


class UserCreateParams(TypedDict, total=False):
    agent_id: Required[str]
    """The agent ID of the user memory."""

    memory: Required[str]
    """The memory of the user memory."""

    context: Optional[str]
    """The context of the user memory."""

    user_prompt: Optional[str]
    """The user prompt of the user memory."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["DataViewListParams"]


class DataViewListParams(TypedDict, total=False):
    service_account_id: Required[str]
    """The id of the service account to list data views for."""

    knowledge_id: str
    """The id of the knowledge to list data views for."""

    limit: int
    """The limit on the number of objects to return, ranging between 1 and 100."""

    offset: int
    """A cursor to use in pagination.

    `offset` is an integer that defines your place in the list. For example, if you
    make a list request and receive 100 objects, starting with `obj_bar`, your
    subsequent call can include `offset=100` to fetch the next page of the list.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["FileObject", "Credits"]


class Credits(BaseModel):
    """Credit consumption for this file upload. `null` when the billing lookup fails."""

    consumed: float
    """The number of credits consumed by the operation."""


class FileObject(BaseModel):
    """The `File` object represents a document that has been uploaded to Datagrid."""

    id: str
    """The file identifier, which can be referenced in the API endpoints."""

    created_at: datetime
    """The ISO string for when the file was created."""

    filename: str
    """The name of the file"""

    media_type: str
    """The media type of the file."""

    object: Literal["file"]
    """The object type, which is always `file`."""

    credits: Optional[Credits] = None
    """Credit consumption for this file upload. `null` when the billing lookup fails."""

    expires_at: Optional[datetime] = None
    """
    The ISO timestamp when the file will expire and be automatically deleted, or
    null if the file does not expire.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .data_views import (
    DataViewsResource,
    AsyncDataViewsResource,
    DataViewsResourceWithRawResponse,
    AsyncDataViewsResourceWithRawResponse,
    DataViewsResourceWithStreamingResponse,
    AsyncDataViewsResourceWithStreamingResponse,
)
from .service_accounts import (
    ServiceAccountsResource,
    AsyncServiceAccountsResource,
    ServiceAccountsResourceWithRawResponse,
    AsyncServiceAccountsResourceWithRawResponse,
    ServiceAccountsResourceWithStreamingResponse,
    AsyncServiceAccountsResourceWithStreamingResponse,
)

__all__ = [
    "ServiceAccountsResource",
    "AsyncServiceAccountsResource",
    "ServiceAccountsResourceWithRawResponse",
    "AsyncServiceAccountsResourceWithRawResponse",
    "ServiceAccountsResourceWithStreamingResponse",
    "AsyncServiceAccountsResourceWithStreamingResponse",
    "DataViewsResource",
    "AsyncDataViewsResource",
    "DataViewsResourceWithRawResponse",
    "AsyncDataViewsResourceWithRawResponse",
    "DataViewsResourceWithStreamingResponse",
    "AsyncDataViewsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .credits import (
    CreditsResource,
    AsyncCreditsResource,
    CreditsResourceWithRawResponse,
    AsyncCreditsResourceWithRawResponse,
    CreditsResourceWithStreamingResponse,
    AsyncCreditsResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .mcp_servers import (
    McpServersResource,
    AsyncMcpServersResource,
    McpServersResourceWithRawResponse,
    AsyncMcpServersResourceWithRawResponse,
    McpServersResourceWithStreamingResponse,
    AsyncMcpServersResourceWithStreamingResponse,
)
from .teamspaces.teamspaces import (
    TeamspacesResource,
    AsyncTeamspacesResource,
    TeamspacesResourceWithRawResponse,
    AsyncTeamspacesResourceWithRawResponse,
    TeamspacesResourceWithStreamingResponse,
    AsyncTeamspacesResourceWithStreamingResponse,
)

__all__ = ["OrganizationResource", "AsyncOrganizationResource"]


class OrganizationResource(SyncAPIResource):
    @cached_property
    def users(self) -> UsersResource:
        return UsersResource(self._client)

    @cached_property
    def teamspaces(self) -> TeamspacesResource:
        return TeamspacesResource(self._client)

    @cached_property
    def mcp_servers(self) -> McpServersResource:
        return McpServersResource(self._client)

    @cached_property
    def credits(self) -> CreditsResource:
        return CreditsResource(self._client)

    @cached_property
    def with_raw_response(self) -> OrganizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return OrganizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OrganizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return OrganizationResourceWithStreamingResponse(self)


class AsyncOrganizationResource(AsyncAPIResource):
    @cached_property
    def users(self) -> AsyncUsersResource:
        return AsyncUsersResource(self._client)

    @cached_property
    def teamspaces(self) -> AsyncTeamspacesResource:
        return AsyncTeamspacesResource(self._client)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResource:
        return AsyncMcpServersResource(self._client)

    @cached_property
    def credits(self) -> AsyncCreditsResource:
        return AsyncCreditsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOrganizationResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOrganizationResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOrganizationResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AsyncOrganizationResourceWithStreamingResponse(self)


class OrganizationResourceWithRawResponse:
    def __init__(self, organization: OrganizationResource) -> None:
        self._organization = organization

    @cached_property
    def users(self) -> UsersResourceWithRawResponse:
        return UsersResourceWithRawResponse(self._organization.users)

    @cached_property
    def teamspaces(self) -> TeamspacesResourceWithRawResponse:
        return TeamspacesResourceWithRawResponse(self._organization.teamspaces)

    @cached_property
    def mcp_servers(self) -> McpServersResourceWithRawResponse:
        return McpServersResourceWithRawResponse(self._organization.mcp_servers)

    @cached_property
    def credits(self) -> CreditsResourceWithRawResponse:
        return CreditsResourceWithRawResponse(self._organization.credits)


class AsyncOrganizationResourceWithRawResponse:
    def __init__(self, organization: AsyncOrganizationResource) -> None:
        self._organization = organization

    @cached_property
    def users(self) -> AsyncUsersResourceWithRawResponse:
        return AsyncUsersResourceWithRawResponse(self._organization.users)

    @cached_property
    def teamspaces(self) -> AsyncTeamspacesResourceWithRawResponse:
        return AsyncTeamspacesResourceWithRawResponse(self._organization.teamspaces)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResourceWithRawResponse:
        return AsyncMcpServersResourceWithRawResponse(self._organization.mcp_servers)

    @cached_property
    def credits(self) -> AsyncCreditsResourceWithRawResponse:
        return AsyncCreditsResourceWithRawResponse(self._organization.credits)


class OrganizationResourceWithStreamingResponse:
    def __init__(self, organization: OrganizationResource) -> None:
        self._organization = organization

    @cached_property
    def users(self) -> UsersResourceWithStreamingResponse:
        return UsersResourceWithStreamingResponse(self._organization.users)

    @cached_property
    def teamspaces(self) -> TeamspacesResourceWithStreamingResponse:
        return TeamspacesResourceWithStreamingResponse(self._organization.teamspaces)

    @cached_property
    def mcp_servers(self) -> McpServersResourceWithStreamingResponse:
        return McpServersResourceWithStreamingResponse(self._organization.mcp_servers)

    @cached_property
    def credits(self) -> CreditsResourceWithStreamingResponse:
        return CreditsResourceWithStreamingResponse(self._organization.credits)


class AsyncOrganizationResourceWithStreamingResponse:
    def __init__(self, organization: AsyncOrganizationResource) -> None:
        self._organization = organization

    @cached_property
    def users(self) -> AsyncUsersResourceWithStreamingResponse:
        return AsyncUsersResourceWithStreamingResponse(self._organization.users)

    @cached_property
    def teamspaces(self) -> AsyncTeamspacesResourceWithStreamingResponse:
        return AsyncTeamspacesResourceWithStreamingResponse(self._organization.teamspaces)

    @cached_property
    def mcp_servers(self) -> AsyncMcpServersResourceWithStreamingResponse:
        return AsyncMcpServersResourceWithStreamingResponse(self._organization.mcp_servers)

    @cached_property
    def credits(self) -> AsyncCreditsResourceWithStreamingResponse:
        return AsyncCreditsResourceWithStreamingResponse(self._organization.credits)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rewrite_response import RewriteResponse as RewriteResponse
from .rewrite_rewrite_text_params import RewriteRewriteTextParams as RewriteRewriteTextParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal

import httpx

from ..types import agent_list_params, agent_create_params, agent_update_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from .._utils import path_template, maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncCursorIDPage, AsyncCursorIDPage
from ..types.agent import Agent
from .._base_client import AsyncPaginator, make_request_options

__all__ = ["AgentsResource", "AsyncAgentsResource"]


class AgentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AgentsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        agent_model: Union[Literal["magpie-1.1", "magpie-1.1-flash", "magpie-2.0", "magpie-2.5", "llm-only"], str, None]
        | Omit = omit,
        corpus: Optional[Iterable[agent_create_params.Corpus]] | Omit = omit,
        custom_prompt: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        disabled_tools: Optional[SequenceNotStr[agent_create_params.DisabledTool]] | Omit = omit,
        knowledge_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        llm_model: Union[
            Literal[
                "gemini-3-pro-preview",
                "gemini-3.1-pro-preview",
                "gemini-3-flash-preview",
                "gemini-2.5-pro",
                "gemini-2.5-pro-preview-05-06",
                "gemini-2.5-flash",
                "gemini-2.5-flash-preview-04-17",
                "gemini-2.5-flash-native-audio-preview-12-2025",
                "gemini-2.5-flash-lite",
                "gpt-5",
                "gpt-5.1",
                "gemini-2.0-flash-001",
                "gemini-2.0-flash",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-002",
                "gemini-1.5-flash-002",
                "gemini-1.5-flash-001",
                "chatgpt-4o-latest",
                "gpt-4o",
                "gpt-4",
                "gpt-4-turbo",
                "gpt-4o-mini",
            ],
            str,
            None,
        ]
        | Omit = omit,
        mcp_servers: Optional[Iterable[agent_create_params.McpServer]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        planning_prompt: Optional[str] | Omit = omit,
        system_prompt: Optional[str] | Omit = omit,
        tools: Optional[SequenceNotStr[agent_create_params.Tool]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """
        Create a new agent

        Args:
          agent_model: The agent model determines the processing mode for Converse requests. Each model
              maps to one of three modes available in the Datagrid UI:

              **Agentic mode** (full tool use, planning, and multi-step reasoning):

              - `magpie-2.0` — Default. Agentic model with proactive planning and reasoning.
              - `magpie-2.5` — Beta. Our latest agentic model — faster, more adaptable, and
                built to handle a broader range of real-world tasks.
              - `magpie-1.1` — Previous-generation agentic model.

              **Ask mode** (lightweight, single-turn Q&A):

              - `magpie-1.1-flash` — Fast model optimized for RAG use cases. Only supports the
                `semantic_search` tool. A 400 error will be returned if other tools are
                specified. Structured outputs are not supported.

              **Fastest mode** (direct LLM response, no tool execution):

              - `llm-only` — Runs a direct LLM conversation with no planning or tool calls. A
                400 error will be returned if tools are specified. Structured outputs are not
                supported.

              Can also accept any custom string value for future model versions.

          corpus: Array of corpus items the agent should use during the converse. When omitted,
              all knowledge is used.

          custom_prompt: Use custom prompt to instruct the style and formatting of the agent's response

          description: The description of the agent

          disabled_tools: Array of the agent tools to disable. Disabling is performed after the
              'agent_tools' rules are applied. For example, agent_tools: null and
              disabled_tools: [data_analysis] will enable everything but the data_analysis
              tool. If nothing or [] is provided, nothing is disabled and therefore only the
              agent_tools setting is relevant.

          knowledge_ids: Deprecated, use corpus instead. Array of Knowledge IDs the agent should use
              during the converse. When omitted, all knowledge is used.

          llm_model: The LLM used to generate responses.

          mcp_servers: Registered MCP servers to enable for this agent.

          name: The name of the agent

          planning_prompt: Define the planning strategy your AI Agent should use when breaking down tasks
              and solving problems

          system_prompt: Directs your AI Agent's operational behavior.

          tools: Array of the agent tools to enable. If not provided, or null is provided -
              default tools of the agent are used. If empty list provided - none of the tools
              are used. When connection_id is set for a tool, it will use that specific
              connection instead of the default one.

              **Tool availability by agent model:**

              - **Agentic** (`magpie-2.0`, `magpie-2.5`, `magpie-1.1`): All tools below are
                available.
              - **Ask** (`magpie-1.1-flash`): Only `semantic_search` is supported. Requests
                specifying other tools will be rejected with a 400 error.
              - **Fastest** (`llm-only`): No tools are executed. Requests specifying tools
                will be rejected with a 400 error.

              Knowledge management tools:

              - data_analysis: Answer statistical or analytical questions like "Show my
                quarterly revenue growth"
              - semantic_search: Search knowledge through natural language queries.
              - agent_memory: Agents can remember experiences, conversations and user
                preferences.
              - schema_info: Helps the Agent understand column names and dataset purpose.
                Avoid disabling
              - table_info: Allow the AI Agent to get information about datasets and schemas
              - create_dataset: Agents respond with data tables

              Actions:

              - calendar: Allow the Agent to access and make changes to your Google Calendar
              - schedule_recurring_message_tool: Eliminate busywork such as: "Send a summary
                of today's meetings at 5pm on workdays"

              Data processing tools:

              - data_classification: Agents handle queries like "Label these emails as high,
                medium, or low priority"
              - data_extraction: Helps the agent understand data from other tools. Avoid
                disabling
              - image_detection: Extract information from images using AI
              - pdf_extraction: Extraction of information from PDFs using AI

              Enhanced response tools:

              - connect_data: Agents provide buttons to import data in response to queries
                like "Connect Hubspot"
              - download_data: Agents handle queries like "download the table as CSV"

              Web tools:

              - web_search: Agents search the internet, and provide links to their sources
              - fetch_url: Fetch URL content
              - company_prospect_researcher: Agents provide information about companies
              - people_prospect_researcher: Agents provide information about people

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/agents",
            body=maybe_transform(
                {
                    "agent_model": agent_model,
                    "corpus": corpus,
                    "custom_prompt": custom_prompt,
                    "description": description,
                    "disabled_tools": disabled_tools,
                    "knowledge_ids": knowledge_ids,
                    "llm_model": llm_model,
                    "mcp_servers": mcp_servers,
                    "name": name,
                    "planning_prompt": planning_prompt,
                    "system_prompt": system_prompt,
                    "tools": tools,
                },
                agent_create_params.AgentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Agent,
        )

    def retrieve(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """
        Get details of a specific agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get(
            path_template("/agents/{agent_id}", agent_id=agent_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Agent,
        )

    def update(
        self,
        agent_id: str,
        *,
        agent_model: Union[Literal["magpie-1.1", "magpie-1.1-flash", "magpie-2.0", "magpie-2.5", "llm-only"], str, None]
        | Omit = omit,
        corpus: Optional[Iterable[agent_update_params.Corpus]] | Omit = omit,
        custom_prompt: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        disabled_tools: Optional[SequenceNotStr[agent_update_params.DisabledTool]] | Omit = omit,
        emoji: Optional[str] | Omit = omit,
        knowledge_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        llm_model: Union[
            Literal[
                "gemini-3-pro-preview",
                "gemini-3.1-pro-preview",
                "gemini-3-flash-preview",
                "gemini-2.5-pro",
                "gemini-2.5-pro-preview-05-06",
                "gemini-2.5-flash",
                "gemini-2.5-flash-preview-04-17",
                "gemini-2.5-flash-native-audio-preview-12-2025",
                "gemini-2.5-flash-lite",
                "gpt-5",
                "gpt-5.1",
                "gemini-2.0-flash-001",
                "gemini-2.0-flash",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-002",
                "gemini-1.5-flash-002",
                "gemini-1.5-flash-001",
                "chatgpt-4o-latest",
                "gpt-4o",
                "gpt-4",
                "gpt-4-turbo",
                "gpt-4o-mini",
            ],
            str,
            None,
        ]
        | Omit = omit,
        mcp_servers: Optional[Iterable[agent_update_params.McpServer]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        planning_prompt: Optional[str] | Omit = omit,
        system_prompt: Optional[str] | Omit = omit,
        tools: Optional[SequenceNotStr[agent_update_params.Tool]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """
        Update an agent configuration

        Args:
          agent_model: The agent model determines the processing mode for Converse requests. Each model
              maps to one of three modes available in the Datagrid UI:

              **Agentic mode** (full tool use, planning, and multi-step reasoning):

              - `magpie-2.0` — Default. Agentic model with proactive planning and reasoning.
              - `magpie-2.5` — Beta. Our latest agentic model — faster, more adaptable, and
                built to handle a broader range of real-world tasks.
              - `magpie-1.1` — Previous-generation agentic model.

              **Ask mode** (lightweight, single-turn Q&A):

              - `magpie-1.1-flash` — Fast model optimized for RAG use cases. Only supports the
                `semantic_search` tool. A 400 error will be returned if other tools are
                specified. Structured outputs are not supported.

              **Fastest mode** (direct LLM response, no tool execution):

              - `llm-only` — Runs a direct LLM conversation with no planning or tool calls. A
                400 error will be returned if tools are specified. Structured outputs are not
                supported.

              Can also accept any custom string value for future model versions.

          corpus: Array of corpus items the agent should use during the converse. When omitted,
              all knowledge is used.

          custom_prompt: Use custom prompt to instruct the style and formatting of the agent's response

          description: The description of the agent

          disabled_tools: Array of the agent tools to disable. Disabling is performed after the
              'agent_tools' rules are applied. For example, agent_tools: null and
              disabled_tools: [data_analysis] will enable everything but the data_analysis
              tool. If nothing or [] is provided, nothing is disabled and therefore only the
              agent_tools setting is relevant.

          emoji: The emoji of the agent

          knowledge_ids: Deprecated, use corpus instead. Array of Knowledge IDs the agent should use
              during the converse. When omitted, all knowledge is used.

          llm_model: The LLM used to generate responses.

          mcp_servers: Registered MCP servers to enable for this agent.

          name: The name of the agent

          planning_prompt: Define the planning strategy your AI Agent should use when breaking down tasks
              and solving problems

          system_prompt: Directs your AI Agent's operational behavior.

          tools: Array of the agent tools to enable. If not provided, or null is provided -
              default tools of the agent are used. If empty list provided - none of the tools
              are used. When connection_id is set for a tool, it will use that specific
              connection instead of the default one.

              **Tool availability by agent model:**

              - **Agentic** (`magpie-2.0`, `magpie-2.5`, `magpie-1.1`): All tools below are
                available.
              - **Ask** (`magpie-1.1-flash`): Only `semantic_search` is supported. Requests
                specifying other tools will be rejected with a 400 error.
              - **Fastest** (`llm-only`): No tools are executed. Requests specifying tools
                will be rejected with a 400 error.

              Knowledge management tools:

              - data_analysis: Answer statistical or analytical questions like "Show my
                quarterly revenue growth"
              - semantic_search: Search knowledge through natural language queries.
              - agent_memory: Agents can remember experiences, conversations and user
                preferences.
              - schema_info: Helps the Agent understand column names and dataset purpose.
                Avoid disabling
              - table_info: Allow the AI Agent to get information about datasets and schemas
              - create_dataset: Agents respond with data tables

              Actions:

              - calendar: Allow the Agent to access and make changes to your Google Calendar
              - schedule_recurring_message_tool: Eliminate busywork such as: "Send a summary
                of today's meetings at 5pm on workdays"

              Data processing tools:

              - data_classification: Agents handle queries like "Label these emails as high,
                medium, or low priority"
              - data_extraction: Helps the agent understand data from other tools. Avoid
                disabling
              - image_detection: Extract information from images using AI
              - pdf_extraction: Extraction of information from PDFs using AI

              Enhanced response tools:

              - connect_data: Agents provide buttons to import data in response to queries
                like "Connect Hubspot"
              - download_data: Agents handle queries like "download the table as CSV"

              Web tools:

              - web_search: Agents search the internet, and provide links to their sources
              - fetch_url: Fetch URL content
              - company_prospect_researcher: Agents provide information about companies
              - people_prospect_researcher: Agents provide information about people

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._patch(
            path_template("/agents/{agent_id}", agent_id=agent_id),
            body=maybe_transform(
                {
                    "agent_model": agent_model,
                    "corpus": corpus,
                    "custom_prompt": custom_prompt,
                    "description": description,
                    "disabled_tools": disabled_tools,
                    "emoji": emoji,
                    "knowledge_ids": knowledge_ids,
                    "llm_model": llm_model,
                    "mcp_servers": mcp_servers,
                    "name": name,
                    "planning_prompt": planning_prompt,
                    "system_prompt": system_prompt,
                    "tools": tools,
                },
                agent_update_params.AgentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Agent,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        limit: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorIDPage[Agent]:
        """
        List all agents for the authenticated organization

        Args:
          after: A cursor to use in pagination. `after` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              ending with `obj_foo`, your subsequent call can include `after=obj_foo` to fetch
              the next page of the list.

          before: A cursor to use in pagination. `before` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              starting with `obj_bar`, your subsequent call can include `before=obj_bar` to
              fetch the previous page of the list.

          limit: The limit on the number of objects to return, ranging between 1 and 100.

          search: Optional search string to filter agents by name. Case-insensitive partial match.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/agents",
            page=SyncCursorIDPage[Agent],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "search": search,
                    },
                    agent_list_params.AgentListParams,
                ),
            ),
            model=Agent,
        )

    def delete(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/agents/{agent_id}", agent_id=agent_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncAgentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAgentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAgentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAgentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AsyncAgentsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        agent_model: Union[Literal["magpie-1.1", "magpie-1.1-flash", "magpie-2.0", "magpie-2.5", "llm-only"], str, None]
        | Omit = omit,
        corpus: Optional[Iterable[agent_create_params.Corpus]] | Omit = omit,
        custom_prompt: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        disabled_tools: Optional[SequenceNotStr[agent_create_params.DisabledTool]] | Omit = omit,
        knowledge_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        llm_model: Union[
            Literal[
                "gemini-3-pro-preview",
                "gemini-3.1-pro-preview",
                "gemini-3-flash-preview",
                "gemini-2.5-pro",
                "gemini-2.5-pro-preview-05-06",
                "gemini-2.5-flash",
                "gemini-2.5-flash-preview-04-17",
                "gemini-2.5-flash-native-audio-preview-12-2025",
                "gemini-2.5-flash-lite",
                "gpt-5",
                "gpt-5.1",
                "gemini-2.0-flash-001",
                "gemini-2.0-flash",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-002",
                "gemini-1.5-flash-002",
                "gemini-1.5-flash-001",
                "chatgpt-4o-latest",
                "gpt-4o",
                "gpt-4",
                "gpt-4-turbo",
                "gpt-4o-mini",
            ],
            str,
            None,
        ]
        | Omit = omit,
        mcp_servers: Optional[Iterable[agent_create_params.McpServer]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        planning_prompt: Optional[str] | Omit = omit,
        system_prompt: Optional[str] | Omit = omit,
        tools: Optional[SequenceNotStr[agent_create_params.Tool]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """
        Create a new agent

        Args:
          agent_model: The agent model determines the processing mode for Converse requests. Each model
              maps to one of three modes available in the Datagrid UI:

              **Agentic mode** (full tool use, planning, and multi-step reasoning):

              - `magpie-2.0` — Default. Agentic model with proactive planning and reasoning.
              - `magpie-2.5` — Beta. Our latest agentic model — faster, more adaptable, and
                built to handle a broader range of real-world tasks.
              - `magpie-1.1` — Previous-generation agentic model.

              **Ask mode** (lightweight, single-turn Q&A):

              - `magpie-1.1-flash` — Fast model optimized for RAG use cases. Only supports the
                `semantic_search` tool. A 400 error will be returned if other tools are
                specified. Structured outputs are not supported.

              **Fastest mode** (direct LLM response, no tool execution):

              - `llm-only` — Runs a direct LLM conversation with no planning or tool calls. A
                400 error will be returned if tools are specified. Structured outputs are not
                supported.

              Can also accept any custom string value for future model versions.

          corpus: Array of corpus items the agent should use during the converse. When omitted,
              all knowledge is used.

          custom_prompt: Use custom prompt to instruct the style and formatting of the agent's response

          description: The description of the agent

          disabled_tools: Array of the agent tools to disable. Disabling is performed after the
              'agent_tools' rules are applied. For example, agent_tools: null and
              disabled_tools: [data_analysis] will enable everything but the data_analysis
              tool. If nothing or [] is provided, nothing is disabled and therefore only the
              agent_tools setting is relevant.

          knowledge_ids: Deprecated, use corpus instead. Array of Knowledge IDs the agent should use
              during the converse. When omitted, all knowledge is used.

          llm_model: The LLM used to generate responses.

          mcp_servers: Registered MCP servers to enable for this agent.

          name: The name of the agent

          planning_prompt: Define the planning strategy your AI Agent should use when breaking down tasks
              and solving problems

          system_prompt: Directs your AI Agent's operational behavior.

          tools: Array of the agent tools to enable. If not provided, or null is provided -
              default tools of the agent are used. If empty list provided - none of the tools
              are used. When connection_id is set for a tool, it will use that specific
              connection instead of the default one.

              **Tool availability by agent model:**

              - **Agentic** (`magpie-2.0`, `magpie-2.5`, `magpie-1.1`): All tools below are
                available.
              - **Ask** (`magpie-1.1-flash`): Only `semantic_search` is supported. Requests
                specifying other tools will be rejected with a 400 error.
              - **Fastest** (`llm-only`): No tools are executed. Requests specifying tools
                will be rejected with a 400 error.

              Knowledge management tools:

              - data_analysis: Answer statistical or analytical questions like "Show my
                quarterly revenue growth"
              - semantic_search: Search knowledge through natural language queries.
              - agent_memory: Agents can remember experiences, conversations and user
                preferences.
              - schema_info: Helps the Agent understand column names and dataset purpose.
                Avoid disabling
              - table_info: Allow the AI Agent to get information about datasets and schemas
              - create_dataset: Agents respond with data tables

              Actions:

              - calendar: Allow the Agent to access and make changes to your Google Calendar
              - schedule_recurring_message_tool: Eliminate busywork such as: "Send a summary
                of today's meetings at 5pm on workdays"

              Data processing tools:

              - data_classification: Agents handle queries like "Label these emails as high,
                medium, or low priority"
              - data_extraction: Helps the agent understand data from other tools. Avoid
                disabling
              - image_detection: Extract information from images using AI
              - pdf_extraction: Extraction of information from PDFs using AI

              Enhanced response tools:

              - connect_data: Agents provide buttons to import data in response to queries
                like "Connect Hubspot"
              - download_data: Agents handle queries like "download the table as CSV"

              Web tools:

              - web_search: Agents search the internet, and provide links to their sources
              - fetch_url: Fetch URL content
              - company_prospect_researcher: Agents provide information about companies
              - people_prospect_researcher: Agents provide information about people

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/agents",
            body=await async_maybe_transform(
                {
                    "agent_model": agent_model,
                    "corpus": corpus,
                    "custom_prompt": custom_prompt,
                    "description": description,
                    "disabled_tools": disabled_tools,
                    "knowledge_ids": knowledge_ids,
                    "llm_model": llm_model,
                    "mcp_servers": mcp_servers,
                    "name": name,
                    "planning_prompt": planning_prompt,
                    "system_prompt": system_prompt,
                    "tools": tools,
                },
                agent_create_params.AgentCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Agent,
        )

    async def retrieve(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """
        Get details of a specific agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._get(
            path_template("/agents/{agent_id}", agent_id=agent_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Agent,
        )

    async def update(
        self,
        agent_id: str,
        *,
        agent_model: Union[Literal["magpie-1.1", "magpie-1.1-flash", "magpie-2.0", "magpie-2.5", "llm-only"], str, None]
        | Omit = omit,
        corpus: Optional[Iterable[agent_update_params.Corpus]] | Omit = omit,
        custom_prompt: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        disabled_tools: Optional[SequenceNotStr[agent_update_params.DisabledTool]] | Omit = omit,
        emoji: Optional[str] | Omit = omit,
        knowledge_ids: Optional[SequenceNotStr[str]] | Omit = omit,
        llm_model: Union[
            Literal[
                "gemini-3-pro-preview",
                "gemini-3.1-pro-preview",
                "gemini-3-flash-preview",
                "gemini-2.5-pro",
                "gemini-2.5-pro-preview-05-06",
                "gemini-2.5-flash",
                "gemini-2.5-flash-preview-04-17",
                "gemini-2.5-flash-native-audio-preview-12-2025",
                "gemini-2.5-flash-lite",
                "gpt-5",
                "gpt-5.1",
                "gemini-2.0-flash-001",
                "gemini-2.0-flash",
                "gemini-1.5-pro-001",
                "gemini-1.5-pro-002",
                "gemini-1.5-flash-002",
                "gemini-1.5-flash-001",
                "chatgpt-4o-latest",
                "gpt-4o",
                "gpt-4",
                "gpt-4-turbo",
                "gpt-4o-mini",
            ],
            str,
            None,
        ]
        | Omit = omit,
        mcp_servers: Optional[Iterable[agent_update_params.McpServer]] | Omit = omit,
        name: Optional[str] | Omit = omit,
        planning_prompt: Optional[str] | Omit = omit,
        system_prompt: Optional[str] | Omit = omit,
        tools: Optional[SequenceNotStr[agent_update_params.Tool]] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Agent:
        """
        Update an agent configuration

        Args:
          agent_model: The agent model determines the processing mode for Converse requests. Each model
              maps to one of three modes available in the Datagrid UI:

              **Agentic mode** (full tool use, planning, and multi-step reasoning):

              - `magpie-2.0` — Default. Agentic model with proactive planning and reasoning.
              - `magpie-2.5` — Beta. Our latest agentic model — faster, more adaptable, and
                built to handle a broader range of real-world tasks.
              - `magpie-1.1` — Previous-generation agentic model.

              **Ask mode** (lightweight, single-turn Q&A):

              - `magpie-1.1-flash` — Fast model optimized for RAG use cases. Only supports the
                `semantic_search` tool. A 400 error will be returned if other tools are
                specified. Structured outputs are not supported.

              **Fastest mode** (direct LLM response, no tool execution):

              - `llm-only` — Runs a direct LLM conversation with no planning or tool calls. A
                400 error will be returned if tools are specified. Structured outputs are not
                supported.

              Can also accept any custom string value for future model versions.

          corpus: Array of corpus items the agent should use during the converse. When omitted,
              all knowledge is used.

          custom_prompt: Use custom prompt to instruct the style and formatting of the agent's response

          description: The description of the agent

          disabled_tools: Array of the agent tools to disable. Disabling is performed after the
              'agent_tools' rules are applied. For example, agent_tools: null and
              disabled_tools: [data_analysis] will enable everything but the data_analysis
              tool. If nothing or [] is provided, nothing is disabled and therefore only the
              agent_tools setting is relevant.

          emoji: The emoji of the agent

          knowledge_ids: Deprecated, use corpus instead. Array of Knowledge IDs the agent should use
              during the converse. When omitted, all knowledge is used.

          llm_model: The LLM used to generate responses.

          mcp_servers: Registered MCP servers to enable for this agent.

          name: The name of the agent

          planning_prompt: Define the planning strategy your AI Agent should use when breaking down tasks
              and solving problems

          system_prompt: Directs your AI Agent's operational behavior.

          tools: Array of the agent tools to enable. If not provided, or null is provided -
              default tools of the agent are used. If empty list provided - none of the tools
              are used. When connection_id is set for a tool, it will use that specific
              connection instead of the default one.

              **Tool availability by agent model:**

              - **Agentic** (`magpie-2.0`, `magpie-2.5`, `magpie-1.1`): All tools below are
                available.
              - **Ask** (`magpie-1.1-flash`): Only `semantic_search` is supported. Requests
                specifying other tools will be rejected with a 400 error.
              - **Fastest** (`llm-only`): No tools are executed. Requests specifying tools
                will be rejected with a 400 error.

              Knowledge management tools:

              - data_analysis: Answer statistical or analytical questions like "Show my
                quarterly revenue growth"
              - semantic_search: Search knowledge through natural language queries.
              - agent_memory: Agents can remember experiences, conversations and user
                preferences.
              - schema_info: Helps the Agent understand column names and dataset purpose.
                Avoid disabling
              - table_info: Allow the AI Agent to get information about datasets and schemas
              - create_dataset: Agents respond with data tables

              Actions:

              - calendar: Allow the Agent to access and make changes to your Google Calendar
              - schedule_recurring_message_tool: Eliminate busywork such as: "Send a summary
                of today's meetings at 5pm on workdays"

              Data processing tools:

              - data_classification: Agents handle queries like "Label these emails as high,
                medium, or low priority"
              - data_extraction: Helps the agent understand data from other tools. Avoid
                disabling
              - image_detection: Extract information from images using AI
              - pdf_extraction: Extraction of information from PDFs using AI

              Enhanced response tools:

              - connect_data: Agents provide buttons to import data in response to queries
                like "Connect Hubspot"
              - download_data: Agents handle queries like "download the table as CSV"

              Web tools:

              - web_search: Agents search the internet, and provide links to their sources
              - fetch_url: Fetch URL content
              - company_prospect_researcher: Agents provide information about companies
              - people_prospect_researcher: Agents provide information about people

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._patch(
            path_template("/agents/{agent_id}", agent_id=agent_id),
            body=await async_maybe_transform(
                {
                    "agent_model": agent_model,
                    "corpus": corpus,
                    "custom_prompt": custom_prompt,
                    "description": description,
                    "disabled_tools": disabled_tools,
                    "emoji": emoji,
                    "knowledge_ids": knowledge_ids,
                    "llm_model": llm_model,
                    "mcp_servers": mcp_servers,
                    "name": name,
                    "planning_prompt": planning_prompt,
                    "system_prompt": system_prompt,
                    "tools": tools,
                },
                agent_update_params.AgentUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Agent,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        limit: int | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Agent, AsyncCursorIDPage[Agent]]:
        """
        List all agents for the authenticated organization

        Args:
          after: A cursor to use in pagination. `after` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              ending with `obj_foo`, your subsequent call can include `after=obj_foo` to fetch
              the next page of the list.

          before: A cursor to use in pagination. `before` is an object ID that defines your place
              in the list. For example, if you make a list request and receive 100 objects,
              starting with `obj_bar`, your subsequent call can include `before=obj_bar` to
              fetch the previous page of the list.

          limit: The limit on the number of objects to return, ranging between 1 and 100.

          search: Optional search string to filter agents by name. Case-insensitive partial match.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/agents",
            page=AsyncCursorIDPage[Agent],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "search": search,
                    },
                    agent_list_params.AgentListParams,
                ),
            ),
            model=Agent,
        )

    async def delete(
        self,
        agent_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/agents/{agent_id}", agent_id=agent_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AgentsResourceWithRawResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

        self.create = to_raw_response_wrapper(
            agents.create,
        )
        self.retrieve = to_raw_response_wrapper(
            agents.retrieve,
        )
        self.update = to_raw_response_wrapper(
            agents.update,
        )
        self.list = to_raw_response_wrapper(
            agents.list,
        )
        self.delete = to_raw_response_wrapper(
            agents.delete,
        )


class AsyncAgentsResourceWithRawResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

        self.create = async_to_raw_response_wrapper(
            agents.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            agents.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            agents.update,
        )
        self.list = async_to_raw_response_wrapper(
            agents.list,
        )
        self.delete = async_to_raw_response_wrapper(
            agents.delete,
        )


class AgentsResourceWithStreamingResponse:
    def __init__(self, agents: AgentsResource) -> None:
        self._agents = agents

        self.create = to_streamed_response_wrapper(
            agents.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            agents.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            agents.update,
        )
        self.list = to_streamed_response_wrapper(
            agents.list,
        )
        self.delete = to_streamed_response_wrapper(
            agents.delete,
        )


class AsyncAgentsResourceWithStreamingResponse:
    def __init__(self, agents: AsyncAgentsResource) -> None:
        self._agents = agents

        self.create = async_to_streamed_response_wrapper(
            agents.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            agents.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            agents.update,
        )
        self.list = async_to_streamed_response_wrapper(
            agents.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            agents.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .user_memory import UserMemory

__all__ = ["UserListResponse"]


class UserListResponse(BaseModel):
    data: List[UserMemory]
    """
    An array containing the actual response elements, paginated by any request
    parameters.
    """

    object: Literal["list"]

    has_more: Optional[bool] = None
    """Whether or not there are more elements available after this set.

    If false, this set comprises the end of the list.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .table import Table as Table
from .knowledge import Knowledge as Knowledge
from .table_list_params import TableListParams as TableListParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from datagrid_ai import Datagrid, AsyncDatagrid
from tests.utils import assert_matches_type
from datagrid_ai.types import Page
from datagrid_ai.pagination import SyncCursorIDPage, AsyncCursorIDPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])
    skip_list = pytest.mark.skip("Prism does not correctly validate oneOf query parameter schemas; all list endpoint requests are rejected regardless of whether `parent` is provided")

    @parametrize
    def test_method_create(self, client: Datagrid) -> None:
        page = client.pages.create(
            name="name",
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Datagrid) -> None:
        page = client.pages.create(
            name="name",
            parent={
                "page_id": "page_id",
                "type": "page",
            },
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Datagrid) -> None:
        response = client.pages.with_raw_response.create(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = response.parse()
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Datagrid) -> None:
        with client.pages.with_streaming_response.create(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = response.parse()
            assert_matches_type(Page, page, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Datagrid) -> None:
        page = client.pages.retrieve(
            "page_id",
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Datagrid) -> None:
        response = client.pages.with_raw_response.retrieve(
            "page_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = response.parse()
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Datagrid) -> None:
        with client.pages.with_streaming_response.retrieve(
            "page_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = response.parse()
            assert_matches_type(Page, page, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Datagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `page_id` but received ''"):
            client.pages.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Datagrid) -> None:
        page = client.pages.update(
            page_id="page_id",
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: Datagrid) -> None:
        page = client.pages.update(
            page_id="page_id",
            name="name",
            parent={
                "page_id": "page_id",
                "type": "page",
            },
            scope="teamspace",
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Datagrid) -> None:
        response = client.pages.with_raw_response.update(
            page_id="page_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = response.parse()
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Datagrid) -> None:
        with client.pages.with_streaming_response.update(
            page_id="page_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = response.parse()
            assert_matches_type(Page, page, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Datagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `page_id` but received ''"):
            client.pages.with_raw_response.update(
                page_id="",
            )

    @parametrize
    @skip_list
    def test_method_list(self, client: Datagrid) -> None:
        page = client.pages.list()
        assert_matches_type(SyncCursorIDPage[Page], page, path=["response"])

    @parametrize
    @skip_list
    def test_method_list_with_all_params(self, client: Datagrid) -> None:
        page = client.pages.list(
            after="after",
            before="before",
            limit=1,
            parent={
                "page_id": "page_id",
                "type": "page",
            },
        )
        assert_matches_type(SyncCursorIDPage[Page], page, path=["response"])

    @parametrize
    @skip_list
    def test_raw_response_list(self, client: Datagrid) -> None:
        response = client.pages.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = response.parse()
        assert_matches_type(SyncCursorIDPage[Page], page, path=["response"])

    @parametrize
    @skip_list
    def test_streaming_response_list(self, client: Datagrid) -> None:
        with client.pages.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = response.parse()
            assert_matches_type(SyncCursorIDPage[Page], page, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Datagrid) -> None:
        page = client.pages.delete(
            "page_id",
        )
        assert page is None

    @parametrize
    def test_raw_response_delete(self, client: Datagrid) -> None:
        response = client.pages.with_raw_response.delete(
            "page_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = response.parse()
        assert page is None

    @parametrize
    def test_streaming_response_delete(self, client: Datagrid) -> None:
        with client.pages.with_streaming_response.delete(
            "page_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = response.parse()
            assert page is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Datagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `page_id` but received ''"):
            client.pages.with_raw_response.delete(
                "",
            )


class TestAsyncPages:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )
    skip_list = pytest.mark.skip("Prism does not correctly validate oneOf query parameter schemas; all list endpoint requests are rejected regardless of whether `parent` is provided")

    @parametrize
    async def test_method_create(self, async_client: AsyncDatagrid) -> None:
        page = await async_client.pages.create(
            name="name",
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncDatagrid) -> None:
        page = await async_client.pages.create(
            name="name",
            parent={
                "page_id": "page_id",
                "type": "page",
            },
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.pages.with_raw_response.create(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = await response.parse()
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncDatagrid) -> None:
        async with async_client.pages.with_streaming_response.create(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = await response.parse()
            assert_matches_type(Page, page, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncDatagrid) -> None:
        page = await async_client.pages.retrieve(
            "page_id",
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.pages.with_raw_response.retrieve(
            "page_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = await response.parse()
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncDatagrid) -> None:
        async with async_client.pages.with_streaming_response.retrieve(
            "page_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = await response.parse()
            assert_matches_type(Page, page, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncDatagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `page_id` but received ''"):
            await async_client.pages.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncDatagrid) -> None:
        page = await async_client.pages.update(
            page_id="page_id",
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncDatagrid) -> None:
        page = await async_client.pages.update(
            page_id="page_id",
            name="name",
            parent={
                "page_id": "page_id",
                "type": "page",
            },
            scope="teamspace",
        )
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.pages.with_raw_response.update(
            page_id="page_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = await response.parse()
        assert_matches_type(Page, page, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncDatagrid) -> None:
        async with async_client.pages.with_streaming_response.update(
            page_id="page_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = await response.parse()
            assert_matches_type(Page, page, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncDatagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `page_id` but received ''"):
            await async_client.pages.with_raw_response.update(
                page_id="",
            )

    @parametrize
    @skip_list
    async def test_method_list(self, async_client: AsyncDatagrid) -> None:
        page = await async_client.pages.list()
        assert_matches_type(AsyncCursorIDPage[Page], page, path=["response"])

    @parametrize
    @skip_list
    async def test_method_list_with_all_params(self, async_client: AsyncDatagrid) -> None:
        page = await async_client.pages.list(
            after="after",
            before="before",
            limit=1,
            parent={
                "page_id": "page_id",
                "type": "page",
            },
        )
        assert_matches_type(AsyncCursorIDPage[Page], page, path=["response"])

    @parametrize
    @skip_list
    async def test_raw_response_list(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.pages.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = await response.parse()
        assert_matches_type(AsyncCursorIDPage[Page], page, path=["response"])

    @parametrize
    @skip_list
    async def test_streaming_response_list(self, async_client: AsyncDatagrid) -> None:
        async with async_client.pages.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = await response.parse()
            assert_matches_type(AsyncCursorIDPage[Page], page, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncDatagrid) -> None:
        page = await async_client.pages.delete(
            "page_id",
        )
        assert page is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncDatagrid) -> None:
        response = await async_client.pages.with_raw_response.delete(
            "page_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        page = await response.parse()
        assert page is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncDatagrid) -> None:
        async with async_client.pages.with_streaming_response.delete(
            "page_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            page = await response.parse()
            assert page is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncDatagrid) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `page_id` but received ''"):
            await async_client.pages.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["KnowledgeListParams", "Parent", "ParentParentPage", "ParentRootPage"]


class KnowledgeListParams(TypedDict, total=False):
    after: str
    """A cursor to use in pagination.

    `after` is an object ID that defines your place in the list. For example, if you
    make a list request and receive 100 objects, ending with `obj_foo`, your
    subsequent call can include `after=obj_foo` to fetch the next page of the list.
    """

    before: str
    """A cursor to use in pagination.

    `before` is an object ID that defines your place in the list. For example, if
    you make a list request and receive 100 objects, starting with `obj_bar`, your
    subsequent call can include `before=obj_bar` to fetch the previous page of the
    list.
    """

    limit: int
    """The limit on the number of objects to return, ranging between 1 and 100."""

    parent: Parent
    """Filter by parent.

    Pass `{"type":"root"}` to get root-level items, or
    `{"type":"page","page_id":"page_123"}` to get items nested under a specific
    page. If not specified, returns all items.
    """


class ParentParentPage(TypedDict, total=False):
    """The parent page reference, indicating where this page is nested"""

    page_id: Required[str]
    """The ID of the parent page. Required when type is 'page'"""

    type: Required[Literal["page"]]
    """The type of parent. 'page' indicates nested under a specific page"""


class ParentRootPage(TypedDict, total=False):
    """The root level object"""

    type: Required[Literal["root"]]
    """The type of parent. 'root' indicates at the root level"""


Parent: TypeAlias = Union[ParentParentPage, ParentRootPage]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["SearchTreeResult", "Credits"]


class Credits(BaseModel):
    """Credit consumption for this search.

    When present, `consumed` reflects the actual variable cost of the retrieval work performed for this request. `null` when the billing write fails or the request is aborted before billing completes.
    """

    consumed: float
    """The number of credits consumed by the operation."""


class SearchTreeResult(BaseModel):
    """
    Hierarchical search results grouped by source (datasets, files, pages) with navigation items.
    """

    credits: Optional[Credits] = None
    """Credit consumption for this search.

    When present, `consumed` reflects the actual variable cost of the retrieval work
    performed for this request. `null` when the billing write fails or the request
    is aborted before billing completes.
    """

    data: List[object]
    """Tree nodes representing search results grouped by source.

    Each node contains the matched content, metadata, score, and child nodes for
    hierarchical display.
    """

    has_more: bool
    """Whether more results are available beyond this page.

    Use the `next` cursor to fetch additional results.
    """

    nav_items: List[object]
    """
    Navigation items providing quick links to the datasets, tables, and files that
    matched the query. Each item includes a name, emoji, URL, and type.
    """

    object: Literal["search_tree"]
    """The object type, always `search_tree`."""

    query: str
    """The original search query."""

    next: Optional[str] = None
    """Pagination cursor (numeric offset as string).

    Pass this value as the `next` query parameter to `/search/tree` to fetch the
    next page.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .page import Page as Page
from .tool import Tool as Tool
from .agent import Agent as Agent
from .secret import Secret as Secret
from .identity import Identity as Identity
from .tool_def import ToolDef as ToolDef
from .ai_source import AISource as AISource
from .connector import Connector as Connector
from .data_view import DataView as DataView
from .knowledge import Knowledge as Knowledge
from .tool_name import ToolName as ToolName
from .connection import Connection as Connection
from .tool_param import ToolParam as ToolParam
from .file_object import FileObject as FileObject
from .conversation import Conversation as Conversation
from .row_metadata import RowMetadata as RowMetadata
from .table_metadata import TableMetadata as TableMetadata
from .file_list_params import FileListParams as FileListParams
from .message_metadata import MessageMetadata as MessageMetadata
from .page_list_params import PageListParams as PageListParams
from .search_ai_result import SearchAIResult as SearchAIResult
from .tool_list_params import ToolListParams as ToolListParams
from .agent_list_params import AgentListParams as AgentListParams
from .converse_response import ConverseResponse as ConverseResponse
from .file_create_params import FileCreateParams as FileCreateParams
from .file_update_params import FileUpdateParams as FileUpdateParams
from .i_frame_event_type import IFrameEventType as IFrameEventType
from .identity_teamspace import IdentityTeamspace as IdentityTeamspace
from .knowledge_metadata import KnowledgeMetadata as KnowledgeMetadata
from .page_create_params import PageCreateParams as PageCreateParams
from .page_update_params import PageUpdateParams as PageUpdateParams
from .search_result_item import SearchResultItem as SearchResultItem
from .search_tree_result import SearchTreeResult as SearchTreeResult
from .secret_list_params import SecretListParams as SecretListParams
from .agent_create_params import AgentCreateParams as AgentCreateParams
from .agent_update_params import AgentUpdateParams as AgentUpdateParams
from .attachment_metadata import AttachmentMetadata as AttachmentMetadata
from .connection_provider import ConnectionProvider as ConnectionProvider
from .search_search_params import SearchSearchParams as SearchSearchParams
from .secret_create_params import SecretCreateParams as SecretCreateParams
from .connector_list_params import ConnectorListParams as ConnectorListParams
from .data_view_list_params import DataViewListParams as DataViewListParams
from .knowledge_list_params import KnowledgeListParams as KnowledgeListParams
from .redirect_url_response import RedirectURLResponse as RedirectURLResponse
from .client_converse_params import ClientConverseParams as ClientConverseParams
from .connection_list_params import ConnectionListParams as ConnectionListParams
from .search_result_resource import SearchResultResource as SearchResultResource
from .search_search_response import SearchSearchResponse as SearchSearchResponse
from .voice_session_response import VoiceSessionResponse as VoiceSessionResponse
from .conversation_sort_field import ConversationSortField as ConversationSortField
from .data_view_create_params import DataViewCreateParams as DataViewCreateParams
from .data_view_list_response import DataViewListResponse as DataViewListResponse
from .knowledge_create_params import KnowledgeCreateParams as KnowledgeCreateParams
from .knowledge_update_params import KnowledgeUpdateParams as KnowledgeUpdateParams
from .search_search_ai_params import SearchSearchAIParams as SearchSearchAIParams
from .connection_create_params import ConnectionCreateParams as ConnectionCreateParams
from .connection_update_params import ConnectionUpdateParams as ConnectionUpdateParams
from .conversation_list_params import ConversationListParams as ConversationListParams
from .knowledge_connect_params import KnowledgeConnectParams as KnowledgeConnectParams
from .data_view_create_response import DataViewCreateResponse as DataViewCreateResponse
from .search_search_tree_params import SearchSearchTreeParams as SearchSearchTreeParams
from .conversation_create_params import ConversationCreateParams as ConversationCreateParams
from .conversation_update_params import ConversationUpdateParams as ConversationUpdateParams
from .voice_start_session_params import VoiceStartSessionParams as VoiceStartSessionParams
from .search_result_resource_type import SearchResultResourceType as SearchResultResourceType
from .connection_provider_list_params import ConnectionProviderListParams as ConnectionProviderListParams
from .connection_provider_create_params import ConnectionProviderCreateParams as ConnectionProviderCreateParams
from .connection_provider_update_params import ConnectionProviderUpdateParams as ConnectionProviderUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import is_given, path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._constants import DEFAULT_TIMEOUT
from ..._base_client import make_request_options
from ...types.memory import user_list_params, user_create_params
from ...types.memory.user_list_response import UserListResponse
from ...types.memory.user_create_response import UserCreateResponse

__all__ = ["UserResource", "AsyncUserResource"]


class UserResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return UserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return UserResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        agent_id: str,
        memory: str,
        context: Optional[str] | Omit = omit,
        user_prompt: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserCreateResponse:
        """Create a user memory.

        This endpoint requires credits and meters the actual
        embedding work performed for the request. The response includes
        `credits.consumed` with the billed amount, or `null` if the billing write fails
        after the memory is successfully created.

        Args:
          agent_id: The agent ID of the user memory.

          memory: The memory of the user memory.

          context: The context of the user memory.

          user_prompt: The user prompt of the user memory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not is_given(timeout) and self._client.timeout == DEFAULT_TIMEOUT:
            timeout = 120
        return self._post(
            "/user-memories",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "memory": memory,
                    "context": context,
                    "user_prompt": user_prompt,
                },
                user_create_params.UserCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserCreateResponse,
        )

    def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListResponse:
        """
        List the memories for a given user and agent that the user has access to

        Args:
          limit: The limit on the number of objects to return, ranging between 1 and 100.

          offset: A cursor to use in pagination. `offset` is an integer that defines your place in
              the list. For example, if you make a list request and receive 100 objects,
              starting with `obj_bar`, your subsequent call can include `offset=100` to fetch
              the next page of the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/user-memories",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            cast_to=UserListResponse,
        )

    def delete(
        self,
        user_memory_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a user memory

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_memory_id:
            raise ValueError(f"Expected a non-empty value for `user_memory_id` but received {user_memory_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/user-memories/{user_memory_id}", user_memory_id=user_memory_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncUserResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUserResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUserResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AsyncUserResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        agent_id: str,
        memory: str,
        context: Optional[str] | Omit = omit,
        user_prompt: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserCreateResponse:
        """Create a user memory.

        This endpoint requires credits and meters the actual
        embedding work performed for the request. The response includes
        `credits.consumed` with the billed amount, or `null` if the billing write fails
        after the memory is successfully created.

        Args:
          agent_id: The agent ID of the user memory.

          memory: The memory of the user memory.

          context: The context of the user memory.

          user_prompt: The user prompt of the user memory.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not is_given(timeout) and self._client.timeout == DEFAULT_TIMEOUT:
            timeout = 120
        return await self._post(
            "/user-memories",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "memory": memory,
                    "context": context,
                    "user_prompt": user_prompt,
                },
                user_create_params.UserCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserCreateResponse,
        )

    async def list(
        self,
        *,
        limit: int | Omit = omit,
        offset: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListResponse:
        """
        List the memories for a given user and agent that the user has access to

        Args:
          limit: The limit on the number of objects to return, ranging between 1 and 100.

          offset: A cursor to use in pagination. `offset` is an integer that defines your place in
              the list. For example, if you make a list request and receive 100 objects,
              starting with `obj_bar`, your subsequent call can include `offset=100` to fetch
              the next page of the list.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/user-memories",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "limit": limit,
                        "offset": offset,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            cast_to=UserListResponse,
        )

    async def delete(
        self,
        user_memory_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a user memory

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not user_memory_id:
            raise ValueError(f"Expected a non-empty value for `user_memory_id` but received {user_memory_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/user-memories/{user_memory_id}", user_memory_id=user_memory_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class UserResourceWithRawResponse:
    def __init__(self, user: UserResource) -> None:
        self._user = user

        self.create = to_raw_response_wrapper(
            user.create,
        )
        self.list = to_raw_response_wrapper(
            user.list,
        )
        self.delete = to_raw_response_wrapper(
            user.delete,
        )


class AsyncUserResourceWithRawResponse:
    def __init__(self, user: AsyncUserResource) -> None:
        self._user = user

        self.create = async_to_raw_response_wrapper(
            user.create,
        )
        self.list = async_to_raw_response_wrapper(
            user.list,
        )
        self.delete = async_to_raw_response_wrapper(
            user.delete,
        )


class UserResourceWithStreamingResponse:
    def __init__(self, user: UserResource) -> None:
        self._user = user

        self.create = to_streamed_response_wrapper(
            user.create,
        )
        self.list = to_streamed_response_wrapper(
            user.list,
        )
        self.delete = to_streamed_response_wrapper(
            user.delete,
        )


class AsyncUserResourceWithStreamingResponse:
    def __init__(self, user: AsyncUserResource) -> None:
        self._user = user

        self.create = async_to_streamed_response_wrapper(
            user.create,
        )
        self.list = async_to_streamed_response_wrapper(
            user.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            user.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rewrite import (
    RewriteResource,
    AsyncRewriteResource,
    RewriteResourceWithRawResponse,
    AsyncRewriteResourceWithRawResponse,
    RewriteResourceWithStreamingResponse,
    AsyncRewriteResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["BetaResource", "AsyncBetaResource"]


class BetaResource(SyncAPIResource):
    @cached_property
    def rewrite(self) -> RewriteResource:
        return RewriteResource(self._client)

    @cached_property
    def with_raw_response(self) -> BetaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return BetaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BetaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return BetaResourceWithStreamingResponse(self)


class AsyncBetaResource(AsyncAPIResource):
    @cached_property
    def rewrite(self) -> AsyncRewriteResource:
        return AsyncRewriteResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBetaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBetaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBetaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/DatagridAI/datagrid-python#with_streaming_response
        """
        return AsyncBetaResourceWithStreamingResponse(self)


class BetaResourceWithRawResponse:
    def __init__(self, beta: BetaResource) -> None:
        self._beta = beta

    @cached_property
    def rewrite(self) -> RewriteResourceWithRawResponse:
        return RewriteResourceWithRawResponse(self._beta.rewrite)


class AsyncBetaResourceWithRawResponse:
    def __init__(self, beta: AsyncBetaResource) -> None:
        self._beta = beta

    @cached_property
    def rewrite(self) -> AsyncRewriteResourceWithRawResponse:
        return AsyncRewriteResourceWithRawResponse(self._beta.rewrite)


class BetaResourceWithStreamingResponse:
    def __init__(self, beta: BetaResource) -> None:
        self._beta = beta

    @cached_property
    def rewrite(self) -> RewriteResourceWithStreamingResponse:
        return RewriteResourceWithStreamingResponse(self._beta.rewrite)


class AsyncBetaResourceWithStreamingResponse:
    def __init__(self, beta: AsyncBetaResource) -> None:
        self._beta = beta

    @cached_property
    def rewrite(self) -> AsyncRewriteResourceWithStreamingResponse:
        return AsyncRewriteResourceWithStreamingResponse(self._beta.rewrite)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .._types import FileTypes, SequenceNotStr

__all__ = ["KnowledgeCreateParams", "Parent", "ParentParentPage", "ParentRootPage"]


class KnowledgeCreateParams(TypedDict, total=False):
    files: Required[SequenceNotStr[FileTypes]]
    """The files to be uploaded and learned.

    Supported media types are `pdf`, `json`, `csv`, `text`, `png`, `jpeg`, `excel`,
    `google sheets`, `docx`, `pptx`.
    """

    name: Optional[str]
    """The name of the knowledge."""

    parent: Optional[Parent]
    """The parent page to nest this knowledge under.

    If not provided, knowledge will be created at the root level.
    """


class ParentParentPage(TypedDict, total=False):
    """The parent page reference, indicating where this page is nested"""

    page_id: Required[str]
    """The ID of the parent page. Required when type is 'page'"""

    type: Required[Literal["page"]]
    """The type of parent. 'page' indicates nested under a specific page"""


class ParentRootPage(TypedDict, total=False):
    """The root level object"""

    type: Required[Literal["root"]]
    """The type of parent. 'root' indicates at the root level"""


Parent: TypeAlias = Union[ParentParentPage, ParentRootPage]
